
# Iris Toolbox for macroeconomic modeling

The Iris Toolbox is a macroeconomic modeling package for
[Matlab](https://www.mathworks.com) developed by the Iris Solutions Team since
2001.

Iris provides tools to support the typical workflows in the theoretical development
and practical operation of macroeconomic models and model-based production frameworks. 
In a seamless command line oriented interface, the toolbox integrates the following four broad areas:

#### [Structural modeling tools](StructuralModeling/index.md)
> 
> Tools for development, implementation, diagnosis, and operation of advanced macro
> models, including nonlinear nonstationary models with forward-looking
> (model-consistent, rational) expectations, or systems of nonlinear
> empirical equations.
> 

#### [Time series modeling](TimeSeriesModeling/index.md)
> 
> Tools for estimation, diagnosis and operation of empirical univariate and
> multivariate time series models with focus on dimension reduction, such
> as VAR, bayesian VAR or dynamic factor models.
> 

#### [Data management](DataManagement/index.md)
> 
> Data management and time series processing functions optimized for use in
> practical macroeconomic models.
> 


#### [Reporting](Reporting/index.md)
> 
> Frameworks for the on-screen and HTML visualization of results.
> 

---

Furthermore, Iris offers a number of utility functions to make
macroeconomic modeling more convenient in Matlab:


#### [Shrinkage estimation](ShrinkageEstimation/index.md)
>
> Shrinkage estimation tools include bayesian techniques with individual
> and system priors for structural models, and prior dummy observations for vector autoregression
> models.
> 


#### [Utilities](Utilities/index.md)
> 
> Utilities to support data visualization and text processing.
> 

